﻿local Engine, Api, Locale, Settings = unpack(Speechless)

local length = Engine.Core.Length
local output = Engine.Core.Output

-- Registration
Api = Engine:RegisterModule("Api", Api)

-- Initialize the module.
function Api:OnInitialize()
end

-- Called when the module is enabled.
function Api:OnEnable()
	local settings = Settings:Get(self.Name)

	if not TalkingHeadFrame then
		TalkingHead_LoadUI()
	end
	
	UIParent:UnregisterEvent("TALKINGHEAD_REQUESTED")
	TalkingHeadFrame:UnregisterEvent("TALKINGHEAD_REQUESTED")

	self:RegisterEvent("TALKINGHEAD_REQUESTED", "OnTalkingHead")

	output(string.format(Locale.API_DEBUG_WELCOME, RAID_CLASS_COLORS[Engine.Class].colorStr, Engine.Name))
end

-- Called when the module is disabled.
function Api:OnDisable()
end

-- Register the default settings for the module.
function Api:OnRegisterDefaults()
	return {
		Enabled = true,
		DisableDialog = true,
		DisableDialogUnlessNew = false,
		DisableAudio = true,
		DisableAudioUnlessNew = false,
		InstancesOnly = false,
		History = {}
	}
end

-- 
function Api:OnTalkingHead()
	local settings = Settings:Get(self.Name)

	if not settings.InstancesOnly or (settings.InstancesOnly and (select(2, IsInInstance()) == "party" or select(2, IsInInstance()) == "raid")) then
		local displayInfo, cameraID, vo, duration, lineNumber, numLines, name, text, isNewTalkingHead = C_TalkingHead.GetCurrentLineInfo()
		local key = string.sub(string.format("%s / %s", name, text), 0, 100)

		-- Determine if the player has seen/heard this dialogue before
		local isNew = settings.History[key] == nil

		local frame = TalkingHeadFrame
		local model = frame.MainFrame.Model

		if frame.finishTimer then
			frame.finishTimer:Cancel()
			frame.finishTimer = nil
		end

		if frame.voHandle then
			StopSound(frame.voHandle)
			frame.voHandle = nil
		end

		local currentDisplayInfo = model:GetDisplayInfo()
		local textFormatted = string.format(text)

		if displayInfo and displayInfo ~= 0 then
			if not settings.DisableDialog or (settings.DisableDialog and settings.DisableDialogUnlessNew and isNew) then
				-- Save this dialogue into the history
				settings.History[key] = true

				frame:Show()

				if currentDisplayInfo ~= displayInfo then
					model.uiCameraID = cameraID
					model:SetDisplayInfo(displayInfo)
				else
					if model.uiCameraID ~= cameraID then
						model.uiCameraID = cameraID
						Model_ApplyUICamera(model, model.uiCameraID)
					end
					TalkingHeadFrame_SetupAnimations(model)
				end

				if isNewTalkingHead then
					TalkingHeadFrame_Reset(frame, textFormatted, name)
					TalkingHeadFrame_FadeinFrames()
				else
					if name ~= frame.NameFrame.Name:GetText() then
						-- Fade out the old name and fade in the new name
						frame.NameFrame.Fadeout:Play()

						C_Timer.After(0.25, function()
							frame.NameFrame.Name:SetText(name)
						end)

						C_Timer.After(0.5, function()
							frame.NameFrame.Fadein:Play()
						end)

						frame.MainFrame.TalkingHeadsInAnim:Play()
					end

					if textFormatted ~= frame.TextFrame.Text:GetText() then
						-- Fade out the old text and fade in the new text
						frame.TextFrame.Fadeout:Play()
				
						C_Timer.After(0.25, function()
							frame.TextFrame.Text:SetText(textFormatted)
						end)

						C_Timer.After(0.5, function()
							frame.TextFrame.Fadein:Play()
						end)
					end
				end
			end

			if not settings.DisableAudio or (settings.DisableAudio and settings.DisableAudioUnlessNew and isNew) then
				-- Save this dialogue into the history
				settings.History[key] = true

				local success, voHandle = PlaySoundKitID(vo, "Talking Head", true, true)
				if success then
					frame.voHandle = voHandle
				end
			end
		end
	end
end